import { formatKeys } from './utils/format_keys.mjs'
import { DynamoDBClient } from "@aws-sdk/client-dynamodb";
import {
    GetItemCommand,
    QueryCommand,
    PutItemCommand,
    DeleteItemCommand,
    DynamoDBDocumentClient
} from "@aws-sdk/lib-dynamodb";

const originalClient = new DynamoDBClient({});
const client = DynamoDBDocumentClient.from(originalClient);


/**
 * Get an item from a DynamoDB table
 */
export async function getDbItem(input, table = process.env.TABLE_NAME) {
    if (!input.sk) {
        throw new Error('Input must have sk defined')
    }

    if (input.pk) {
        const input = {
            TableName: table,
            Key: {
                pk: { S: input.pk },
                sk: { S: input.sk }
            }
        }

        const command = new GetItemCommand(input)
        const item = await client.send(command)
        return item.Item || false
    }

    if (input.pk2) {
        const input = {
            TableName: table,
            IndexName: 'pk2',
            Key: {
                pk2: input.pk2,
                sk: input.sk
            }
        }

        const command = new GetItemCommand(input)
        const item = await client.send(command)
        return item.Item || false
    }
    if (input.pk3) {
        const input = {
            TableName: table,
            IndexName: 'pk3',
            Key: {
                pk3: input.pk3,
                sk: input.sk
            }
        }

        const command = new GetItemCommand(input)
        const item = await client.send(command)
        return item.Item || false
    }

    throw new Error('Input must have pk, pk2, or pk3 defined')
}

/**
 * Query items in a DynamoDB table with begins with
 */
export async function listDbItems(input, table = process.env.TABLE_NAME) {
    if (!input.sk) {
        throw new Error('Input must have sk defined')
    }

    if (!input.pk && !input.pk2 && !input.pk3) {
        throw new Error('Input must have either pk, pk2, or pk3 defined')
    }

    let params = {}

    if (input.pk) {
        params = {
            TableName: table,
            ...(input.limit ? { Limit: input.limit } : {}),
            ...(input.startAt ? { ExclusiveStartKey: input.startAt } : {}),
            KeyConditionExpression: 'pk = :pk AND begins_with(sk, :sk)',
            ExpressionAttributeValues: {
                ':pk': input.pk,
                ':sk': input.sk
            }
        }
    }

    if (input.pk2) {
        params = {
            TableName: table,
            ...(input.limit ? { Limit: input.limit } : {}),
            ...(input.startAt ? { ExclusiveStartKey: input.startAt } : {}),
            IndexName: 'pk2',
            KeyConditionExpression: 'pk2 = :gsi AND begins_with(sk, :sk)',
            ExpressionAttributeValues: {
                ':gsi': input.pk2,
                ':sk': input.sk
            }
        }
    }

    if (input.pk3) {
        params = {
            TableName: table,
            ...(input.limit ? { Limit: input.limit } : {}),
            ...(input.startAt ? { ExclusiveStartKey: input.startAt } : {}),
            IndexName: 'pk3',
            KeyConditionExpression: 'pk3 = :gsi AND begins_with(sk, :sk)',
            ExpressionAttributeValues: {
                ':gsi': input.pk3,
                ':sk': input.sk
            }
        }
    }

    const command = new QueryCommand(params)
    const result = await client.send(command)

    return result.Items || []
}


/**
 * Put an item into a DynamoDB table.
 * This will overwrite if item already exists in table
 */
export async function setDbItem(input, table = process.env.TABLE_NAME) {
    if (!input.pk && !input.pk2 && !input.pk3) {
        throw new Error('create must have either pk, pk2, or pk3 defined')
    }

    if (!input.sk) {
        throw new Error('create must have a sk defined')
    }

    const formattedInput = formatKeys(input)

    const params = {
        TableName: table,
        Item: formattedInput
    }
    const command = new PutItemCommand(params)
    await client.send(command)

    return formattedInput
}

/**
 * Remove an item from a DynamoDB Tables
 */
export async function removeDbItem(input, table = process.env.TABLE_NAME) {
    const params = {
        TableName: table,
        Key: {
            pk: input.pk,
            sk: input.sk
        }
    }
    const command = new DeleteItemCommand(params)
    await client.send(command)
    return input
}